@@ -173,7 +173,7 @@ panel_applet_frame_create (PanelToplevel *toplevel,
 	client  = panel_gconf_get_client ();
 	profile = panel_profile_get_name ();
 
-	id = panel_profile_prepare_object (PANEL_OBJECT_BONOBO, toplevel, position);
+	id = panel_profile_prepare_object (PANEL_OBJECT_BONOBO, toplevel, position, FALSE);
 
 	key = panel_gconf_full_key (PANEL_GCONF_APPLETS, profile, id, "bonobo_iid");
 	gconf_client_set_string (client, key, iid, NULL);
