@@ -19,8 +19,8 @@
 #include "panel-util.h"
 #include "panel-marshal.h"
 #include "panel-typebuiltins.h"
-#include "drawer-widget.h"
 #include "panel-applet-frame.h"
+#include "panel-globals.h"
 
 #define MOVE_INCREMENT 2
 
@@ -31,12 +31,10 @@ typedef enum {
 } PanelMovementType;
 
 enum {
-	ORIENT_CHANGE_SIGNAL,
 	SIZE_CHANGE_SIGNAL,
 	APPLET_MOVE_SIGNAL,
 	APPLET_ADDED_SIGNAL,
 	APPLET_REMOVED_SIGNAL,
-	BACK_CHANGE_SIGNAL,
 	PUSH_MOVE_SIGNAL,
 	SWITCH_MOVE_SIGNAL,
 	FREE_MOVE_SIGNAL,
@@ -48,19 +46,12 @@ enum {
 
 static guint panel_widget_signals [LAST_SIGNAL] = {0};
 
-GSList *panels = NULL; /*other panels we might want to move the applet to*/
-
 /*define for some debug output*/
 #undef PANEL_WIDGET_DEBUG
 
-/*there  can universally be only one applet being dragged since we assume
-we only have one mouse :) */
-gboolean panel_applet_in_drag = FALSE;
+static gboolean panel_applet_in_drag = FALSE;
 static GtkWidget *saved_focus_widget = NULL;
 
-/* Commie mode! */
-extern gboolean commie_mode;
-
 static void panel_widget_class_init     (PanelWidgetClass *klass);
 static void panel_widget_instance_init  (PanelWidget      *panel_widget);
 static void panel_widget_size_request   (GtkWidget        *widget,
@@ -129,6 +120,15 @@ applet_data_compare (AppletData *ad1, AppletData *ad2)
 	return ad1->pos - ad2->pos;
 }
 
+static void
+emit_applet_moved (PanelWidget *panel_widget,
+		   AppletData  *applet)
+{
+	g_signal_emit (panel_widget,
+		       panel_widget_signals [APPLET_MOVE_SIGNAL], 0,
+		       applet->applet);
+}
+
 /************************
  widget core
  ************************/
@@ -227,7 +227,7 @@ add_all_move_bindings (PanelWidget *panel)
                                       GDK_space, 0,
                                       "end_move", 0);
 
-	focus_widget = gtk_window_get_focus (GTK_WINDOW (panel->panel_parent));
+	focus_widget = gtk_window_get_focus (GTK_WINDOW (panel->toplevel));
 	if (GTK_IS_SOCKET (focus_widget)) {
 		/*
 		 * If the focus widget is a GtkSocket, i.e. the
@@ -319,17 +319,6 @@ panel_widget_class_init (PanelWidgetClass *class)
 	GtkWidgetClass *widget_class = (GtkWidgetClass*) class;
 	GtkContainerClass *container_class = (GtkContainerClass*) class;
 
-	panel_widget_signals[ORIENT_CHANGE_SIGNAL] =
-                g_signal_new ("orient_change",
-                              G_TYPE_FROM_CLASS (object_class),
-                              G_SIGNAL_RUN_LAST,
-                              G_STRUCT_OFFSET (PanelWidgetClass, orient_change),
-                              NULL,
-                              NULL, 
-                              panel_marshal_VOID__VOID,
-                              G_TYPE_NONE,
-                              0);
-
 	panel_widget_signals[SIZE_CHANGE_SIGNAL] =
                 g_signal_new ("size_change",
                               G_TYPE_FROM_CLASS (object_class),
@@ -377,17 +366,6 @@ panel_widget_class_init (PanelWidgetClass *class)
                               1,
                               G_TYPE_POINTER); 
 
-	panel_widget_signals[BACK_CHANGE_SIGNAL] =
-                g_signal_new ("back_change",
-                              G_TYPE_FROM_CLASS (object_class),
-                              G_SIGNAL_RUN_LAST,
-                              G_STRUCT_OFFSET (PanelWidgetClass, back_change),
-                              NULL,
-                              NULL, 
-                              panel_marshal_VOID__VOID,
-                              G_TYPE_NONE,
-                              0);
-
 	panel_widget_signals[PUSH_MOVE_SIGNAL] =
 		g_signal_new ("push_move",
                               G_TYPE_FROM_CLASS (class),
@@ -447,12 +425,10 @@ panel_widget_class_init (PanelWidgetClass *class)
                               G_TYPE_NONE,
                               0);
 
-	class->orient_change = NULL;
 	class->size_change = NULL;
 	class->applet_move = NULL;
 	class->applet_added = NULL;
 	class->applet_removed = NULL;
-	class->back_change = NULL;
 	class->push_move = panel_widget_push_move_applet;
 	class->switch_move = panel_widget_switch_move_applet;
 	class->free_move = panel_widget_free_move_applet;
@@ -590,8 +566,7 @@ panel_widget_cadd (GtkContainer *container,
 	g_return_if_fail (PANEL_IS_WIDGET (container));
 	g_return_if_fail (GTK_IS_WIDGET (widget));
 
-	panel_widget_add (PANEL_WIDGET (container), widget, 0,
-			  FALSE, FALSE, FALSE);
+	panel_widget_add (PANEL_WIDGET (container), widget, 0, FALSE);
 
 	p = g_object_get_data (G_OBJECT(widget),
 			       PANEL_APPLET_ASSOC_PANEL_KEY);
@@ -627,11 +602,8 @@ panel_widget_cremove (GtkContainer *container, GtkWidget *widget)
 	if (GTK_CONTAINER_CLASS (panel_widget_parent_class)->remove)
 		(* GTK_CONTAINER_CLASS (panel_widget_parent_class)->remove) (container,
 								widget);
-	if (ad != NULL) {
-		PanelWidget *panel = PANEL_WIDGET (container);
-
+	if (ad)
 		panel->applet_list = g_list_remove (panel->applet_list, ad);
-	}
 
 	g_signal_emit (G_OBJECT (container),
 		       panel_widget_signals[APPLET_REMOVED_SIGNAL],
@@ -664,32 +636,33 @@ get_applet_list_pos (PanelWidget *panel,
 
 /*tells us if an applet is "stuck" on the right side*/
 int
-panel_widget_is_applet_stuck(PanelWidget *panel, GtkWidget *applet)
+panel_widget_is_applet_stuck (PanelWidget *panel_widget,
+			      GtkWidget   *widget)
 {
-	AppletData *ad;
-	GList *list;
-	int i;
+	AppletData *applet;
 
-	g_return_val_if_fail(PANEL_IS_WIDGET(panel),FALSE);
-	g_return_val_if_fail(GTK_IS_WIDGET(applet),FALSE);
+	g_return_val_if_fail (PANEL_IS_WIDGET (panel_widget), FALSE);
+	g_return_val_if_fail (GTK_IS_WIDGET (widget), FALSE);
 
-	ad = g_object_get_data (G_OBJECT(applet), PANEL_APPLET_DATA);
+	applet = g_object_get_data (G_OBJECT (widget), PANEL_APPLET_DATA);
+	if (applet) {
+		GList *applet_list, *l;
+		int    end_pos = -1;
+	
+		applet_list = g_list_find (panel_widget->applet_list, applet);
 
-	g_return_val_if_fail(ad!=NULL,FALSE);
+		for (l = applet_list; l; l = l->next) {
+			applet = l->data;
 
-	list = g_list_find(panel->applet_list,ad);
+			if (end_pos != -1 && applet->pos != end_pos)
+				break;
 
-	g_return_val_if_fail(list!=NULL,FALSE);
+			end_pos = applet->pos + applet->cells;
+			if (end_pos >= panel_widget->size)
+				return TRUE;
+		}
+	}
 
-	do {
-		i=ad->pos+ad->cells;
-		if(i==panel->size)
-			return TRUE;
-		list = g_list_next(list);
-		if(!list)
-			break;
-		ad = list->data;
-	} while(ad->pos==i);
 	return FALSE;
 }
 
@@ -711,7 +684,7 @@ get_size_from_hints (AppletData *ad, int cells)
 		}
 	}
 	
-	return cells;
+	return MAX (cells, ad->min_cells);
 }
 
 static int
@@ -743,10 +716,10 @@ allocate_dirty_child(gpointer data)
 		if (panel->packed && ad->expand_major && ad->size_hints)
 			challoc.width = CLAMP (MAX (ad->size_hints [0], chreq.width),
 					       chreq.width,
-					       GTK_WIDGET (panel)->allocation.width - ad->pos);
+					       GTK_WIDGET (panel)->allocation.width - ad->constrained);
 
 		ad->cells = challoc.width; 
-		challoc.x = ad->pos;
+		challoc.x = ad->constrained;
 		challoc.y = GTK_WIDGET(panel)->allocation.height / 2 -
 			     challoc.height / 2;
 	} else {
@@ -756,51 +729,40 @@ allocate_dirty_child(gpointer data)
 		if (panel->packed && ad->expand_major && ad->size_hints)
 			challoc.height = CLAMP (MAX (ad->size_hints [0], chreq.height),
 						chreq.height,
-						GTK_WIDGET (panel)->allocation.height - ad->pos);
+						GTK_WIDGET (panel)->allocation.height - ad->constrained);
 
 		ad->cells = challoc.height;
 		challoc.x = GTK_WIDGET(panel)->allocation.width / 2 -
 			    challoc.width / 2;
-		challoc.y = ad->pos;
+		challoc.y = ad->constrained;
 	}
 
 	ad->min_cells = ad->cells;
 
 	if (!panel->packed && ad->expand_major) {
 		GList *l = g_list_find (panel->applet_list, ad);
-		int cells;
 		
 		if (l) {
 			if (l->next) {
 				nad = l->next->data;
-				cells = nad->pos - ad->pos - 1;
+				ad->cells = nad->constrained - ad->constrained - 1;
 			} else {
-				cells = panel->size - ad->pos - 1;
+				ad->cells = panel->size - ad->constrained - 1;
 			}
 
 			if (ad->size_hints) 
-				cells = get_size_from_hints (ad, cells);
-			
-			cells = MAX (cells, ad->min_cells);
-			
-			ad->cells = cells;
+				ad->cells = get_size_from_hints (ad, ad->cells);
 			
 			if (panel->orient == GTK_ORIENTATION_HORIZONTAL) {
 				challoc.width = ad->cells;
 			} else {
 				challoc.height = ad->cells;
 			}
-				
-
 		}
 	}
 
 	gtk_widget_size_allocate(ad->applet,&challoc);
 
-	g_signal_emit (G_OBJECT(panel),
-		       panel_widget_signals[APPLET_MOVE_SIGNAL],
-		       0, ad->applet);
-
 	return FALSE;
 }
 
@@ -824,24 +786,28 @@ panel_widget_switch_applet_right (PanelWidget *panel,
 	
 	ad = list->data;
 
-	if (ad->pos + ad->min_cells >= panel->size)
+	if (ad->constrained + ad->min_cells >= panel->size)
 		return;
 
 	if (list->next)
 		nad = list->next->data;
 
-	if (!nad || nad->pos > ad->pos + ad->min_cells) {
-		ad->pos += MOVE_INCREMENT;
+	if (!nad || nad->constrained > ad->constrained + ad->min_cells) {
+		ad->pos = ad->constrained += MOVE_INCREMENT;
 		panel_widget_queue_applet_for_resize (ad);
+		emit_applet_moved (panel, ad);
 		return;
 	}
 
-	nad->pos = ad->pos;
-	ad->pos = nad->pos + nad->min_cells;
+	nad->constrained = nad->pos = ad->constrained;
+	ad->constrained = ad->pos = nad->constrained + nad->min_cells;
 	panel->applet_list = panel_g_list_swap_next (panel->applet_list,list);
 
 	panel_widget_queue_applet_for_resize (ad);
 	panel_widget_queue_applet_for_resize (nad);
+
+	emit_applet_moved (panel, ad);
+	emit_applet_moved (panel, nad);
 }
 
 static void
@@ -856,26 +822,30 @@ panel_widget_switch_applet_left (PanelWidget *panel,
 	
 	ad = list->data;
 
-	if (ad->pos <= 0)
+	if (ad->constrained <= 0)
 		return;
 
 	if (list->prev)
 		pad = list->prev->data;
 
-	if (!pad || pad->pos + pad->min_cells < ad->pos) {
-		ad->pos -= MOVE_INCREMENT;
+	if (!pad || pad->constrained + pad->min_cells < ad->constrained) {
+		ad->pos = ad->constrained -= MOVE_INCREMENT;
 		panel_widget_queue_applet_for_resize (ad);
-		if (pad && (pad->pos + pad->cells) >= ad->pos)
+		if (pad && (pad->constrained + pad->cells) >= ad->constrained)
 			panel_widget_queue_applet_for_resize (pad);
+		emit_applet_moved (panel, ad);
 		return;
 	}
 
-	ad->pos = pad->pos;
-	pad->pos = ad->pos + ad->min_cells;
+	ad->constrained = ad->pos = pad->constrained;
+	pad->constrained = pad->pos = ad->constrained + ad->min_cells;
 	panel->applet_list = panel_g_list_swap_prev (panel->applet_list, list);
 
 	panel_widget_queue_applet_for_resize (ad);
 	panel_widget_queue_applet_for_resize (pad);
+
+	emit_applet_moved (panel, ad);
+	emit_applet_moved (panel, pad);
 }
 
 static int
@@ -889,15 +859,14 @@ panel_widget_get_right_switch_pos(PanelWidget *panel, GList *list)
 	
 	ad = list->data;
 
-
 	if(list->next)
 		nad = list->next->data;
 
-	if(!nad || nad->pos > ad->pos+ad->cells) {
-		return ad->pos+1;
+	if(!nad || nad->constrained > ad->constrained+ad->cells) {
+		return ad->constrained+1;
 	}
 
-	return nad->pos+nad->min_cells-ad->cells;
+	return nad->constrained+nad->min_cells-ad->cells;
 }
 
 static int
@@ -913,9 +882,9 @@ panel_widget_get_left_switch_pos(PanelWidget *panel, GList *list)
 
 	if(list->prev)
 		pad = list->prev->data;
-	if(!pad || pad->pos+pad->min_cells < ad->pos)
-		return ad->pos-1;
-	return pad->pos;
+	if(!pad || pad->constrained+pad->min_cells < ad->constrained)
+		return ad->constrained-1;
+	return pad->constrained;
 }
 
 
@@ -936,13 +905,13 @@ panel_widget_switch_move (PanelWidget *panel, AppletData *ad, int moveby)
 	list = g_list_find (panel->applet_list, ad);
 	g_return_if_fail (list != NULL);
 
-	finalpos = ad->pos + moveby;
+	finalpos = ad->constrained + moveby;
 
-	if (ad->pos < finalpos) {
-		while (ad->pos < finalpos) {
+	if (ad->constrained < finalpos) {
+		while (ad->constrained < finalpos) {
 			pos = panel_widget_get_right_switch_pos (panel, list);
 			if (pos + ad->min_cells > panel->size ||
-			    abs (pos - finalpos) >= abs (ad->pos - finalpos))
+			    abs (pos - finalpos) >= abs (ad->constrained - finalpos))
 				break;
 			panel_widget_switch_applet_right (panel, list);
 		}
@@ -952,9 +921,9 @@ panel_widget_switch_move (PanelWidget *panel, AppletData *ad, int moveby)
 				panel_widget_queue_applet_for_resize(pad);
 		}
 	} else {
-		while (ad->pos > finalpos) {
+		while (ad->constrained > finalpos) {
 			pos = panel_widget_get_left_switch_pos (panel, list);
-			if (pos < 0 || abs (pos - finalpos) >= abs (ad->pos - finalpos))
+			if (pos < 0 || abs (pos - finalpos) >= abs (ad->constrained - finalpos))
 				break;
 			panel_widget_switch_applet_left (panel, list);
 		}
@@ -972,16 +941,17 @@ push_applet_right(PanelWidget *panel, GList *list)
 	if (list) {
 		ad = list->data;
 	
-		if(ad->pos + ad->min_cells >= panel->size)
+		if(ad->constrained + ad->min_cells >= panel->size)
 			return FALSE;
 
 		if(list->next)
 			nad = list->next->data;
 	}
 
-	if(!nad || nad->pos > ad->pos+ad->min_cells) {
-		ad->pos++;
+	if(!nad || nad->constrained > ad->constrained+ad->min_cells) {
+		ad->pos = ++ad->constrained;
 		panel_widget_queue_applet_for_resize(ad);
+		emit_applet_moved (panel, ad);
 		return TRUE;
 	}
 
@@ -990,8 +960,9 @@ push_applet_right(PanelWidget *panel, GList *list)
 	if(!push_applet_right(panel,list->next)) {
 		return FALSE;
 	}
-	ad->pos++;
+	ad->pos = ++ad->constrained;
 	panel_widget_queue_applet_for_resize(ad);
+	emit_applet_moved (panel, ad);
 	return TRUE;
 }
 
@@ -1006,18 +977,19 @@ push_applet_left(PanelWidget *panel, GList *list)
 	if (list) {
 		ad = list->data;
 
-		if(ad->pos <= 0)
+		if(ad->constrained <= 0)
 			return FALSE;
 
 		if(list->prev)
 			pad = list->prev->data;
 	}
 
-	if(!list || !pad || pad->pos+pad->min_cells < ad->pos) {
-		ad->pos--;
+	if(!list || !pad || pad->constrained+pad->min_cells < ad->constrained) {
+		ad->pos = --ad->constrained;
 		panel_widget_queue_applet_for_resize(ad);
-		if (pad && pad->pos+pad->cells >= ad->pos)
+		if (pad && pad->constrained+pad->cells >= ad->constrained)
 			panel_widget_queue_applet_for_resize(pad);
+		emit_applet_moved (panel, ad);
 		return TRUE;
 	}
 
@@ -1026,8 +998,9 @@ push_applet_left(PanelWidget *panel, GList *list)
 	if(!push_applet_left(panel,list->prev)) {
 		return FALSE;
 	}
-	ad->pos--;
+	ad->pos = --ad->constrained;
 	panel_widget_queue_applet_for_resize(ad);
+	emit_applet_moved (panel, ad);
 	return TRUE;
 }
 
@@ -1047,10 +1020,10 @@ panel_widget_push_move (PanelWidget *panel, AppletData *ad, int moveby)
 	list = g_list_find (panel->applet_list, ad);
 	g_return_if_fail (list != NULL);
 
-	finalpos = ad->pos + moveby;
+	finalpos = ad->constrained + moveby;
 
-	if (ad->pos < finalpos) {
-		while (ad->pos < finalpos) {
+	if (ad->constrained < finalpos) {
+		while (ad->constrained < finalpos) {
 			if ( ! push_applet_right (panel, list))
 				break;
 		}
@@ -1060,7 +1033,7 @@ panel_widget_push_move (PanelWidget *panel, AppletData *ad, int moveby)
 				panel_widget_queue_applet_for_resize(pad);
 		}
 	} else {
-                while (ad->pos > finalpos) {
+                while (ad->constrained > finalpos) {
 			if ( ! push_applet_left (panel, list))
 				break;
 		}
@@ -1094,20 +1067,18 @@ panel_widget_right_stick(PanelWidget *panel,int old_size)
 
 	ad = list->data;
 	do { 
-		i = ad->pos;
-		ad->pos = pos--;
+		i = ad->constrained;
+		ad->pos = ad->constrained = pos--;
 		ad->cells = 1;
 		prev = list;
 		list = g_list_previous(list);
 		if(!list)
 			break;
 		ad = list->data;
-	} while(ad->pos+ad->cells == i);
+	} while(ad->constrained + ad->cells == i);
 
-	for(list = prev;list!=NULL;list=g_list_next(list))
-		g_signal_emit (G_OBJECT(panel),
-			       panel_widget_signals[APPLET_MOVE_SIGNAL],
-			       0, ((AppletData *)list->data)->applet);
+	for (list = prev; list; list = list->next)
+		emit_applet_moved (panel, list->data);
 }
 
 static void
@@ -1211,8 +1182,6 @@ panel_widget_size_allocate(GtkWidget *widget, GtkAllocation *allocation)
 {
 	PanelWidget *panel;
 	GList *list;
-	GSList *send_move = NULL;
-	GSList *li;
 	int i;
 	int old_size;
 	int old_thick;
@@ -1220,12 +1189,9 @@ panel_widget_size_allocate(GtkWidget *widget, GtkAllocation *allocation)
 
 	g_return_if_fail(PANEL_IS_WIDGET(widget));
 	g_return_if_fail(allocation!=NULL);
-	
+
 	panel = PANEL_WIDGET(widget);
 
-	/* allow drawing if it was inhibited */
-	panel->inhibit_draw = FALSE;
-	
 	old_size = panel->size;
 	old_thick = panel->thick;
 	
@@ -1255,11 +1221,9 @@ panel_widget_size_allocate(GtkWidget *widget, GtkAllocation *allocation)
 			GtkAllocation challoc;
 			GtkRequisition chreq;
 			gtk_widget_get_child_requisition(ad->applet,&chreq);
+
+			ad->constrained = i;
 			
-			if(i != ad->pos) {
-				ad->pos = i;
-				send_move = g_slist_prepend(send_move,ad);
-			}
 			challoc.width = chreq.width;
 			challoc.height = chreq.height;
 			if(panel->orient == GTK_ORIENTATION_HORIZONTAL) {
@@ -1272,7 +1236,7 @@ panel_widget_size_allocate(GtkWidget *widget, GtkAllocation *allocation)
 							       allocation->width - i);
 
 				ad->cells = challoc.width;
-				challoc.x = ad->pos;
+				challoc.x = ad->constrained;
 				challoc.y = allocation->height / 2 - challoc.height / 2;
 			} else {
 				if (ad->expand_minor)
@@ -1285,7 +1249,7 @@ panel_widget_size_allocate(GtkWidget *widget, GtkAllocation *allocation)
 
 				ad->cells = challoc.height;
 				challoc.x = allocation->width / 2 - challoc.width / 2;
-				challoc.y = ad->pos;
+				challoc.y = ad->constrained;
 			}
 			ad->min_cells  = ad->cells;
 			ad->dirty = FALSE;
@@ -1309,9 +1273,11 @@ panel_widget_size_allocate(GtkWidget *widget, GtkAllocation *allocation)
 				ad->cells = chreq.height;
 
 			ad->min_cells = ad->cells;
+
+			ad->constrained = ad->pos;
 			
 			if (ad->expand_major) {
-				cells = (i - ad->pos) - 1;
+				cells = (i - ad->constrained) - 1;
 
 				if (ad->size_hints) 
 					cells = get_size_from_hints (ad, cells);
@@ -1320,11 +1286,10 @@ panel_widget_size_allocate(GtkWidget *widget, GtkAllocation *allocation)
 				ad->cells = cells;
 			}
 
-			if(ad->pos+ad->min_cells > i) {
-				ad->pos = i - ad->min_cells;
-				send_move = g_slist_prepend(send_move,ad);
-			}
-			i = ad->pos;
+			if (ad->constrained + ad->min_cells > i)
+				ad->constrained = i - ad->min_cells;
+
+			i = ad->constrained;
 		}
 		if(i < 0) {
 			i = 0;
@@ -1333,13 +1298,10 @@ panel_widget_size_allocate(GtkWidget *widget, GtkAllocation *allocation)
 			    list = g_list_next(list)) {
 				AppletData *ad = list->data;
 				
-				if(ad->pos < i) {
-					ad->pos = i;
-					if(!g_slist_find(send_move,ad))
-						send_move = g_slist_prepend(send_move,ad);
-				}
+				if (ad->constrained < i)
+					ad->constrained = i;
 				
-				i = ad->pos + ad->cells;
+				i = ad->constrained + ad->cells;
 			}
 		}
 
@@ -1357,7 +1319,7 @@ panel_widget_size_allocate(GtkWidget *widget, GtkAllocation *allocation)
 				if (ad->expand_minor) {
 					challoc.height = allocation->height;
 				}
-				challoc.x = ad->pos;
+				challoc.x = ad->constrained;
 				challoc.y = allocation->height / 2 - challoc.height / 2;
 			} else {
 				challoc.height = ad->cells;
@@ -1365,7 +1327,7 @@ panel_widget_size_allocate(GtkWidget *widget, GtkAllocation *allocation)
 					challoc.width = allocation->width;
 				}
 				challoc.x = allocation->width / 2 - challoc.width / 2;
-				challoc.y = ad->pos;
+				challoc.y = ad->constrained;
 			}
 			ad->dirty = FALSE;
 			
@@ -1377,14 +1339,6 @@ panel_widget_size_allocate(GtkWidget *widget, GtkAllocation *allocation)
 	else
 		panel->thick = allocation->width;
 
-	for(li=send_move;li!=NULL;li=g_slist_next(li)) {
-		AppletData *ad = li->data;
-		g_signal_emit (G_OBJECT(panel),
-			       panel_widget_signals[APPLET_MOVE_SIGNAL],
-			       0, ad->applet);
-	}
-	g_slist_free(send_move);
-
 	panel_widget_set_background_region (panel);
 }
 
@@ -1416,38 +1370,6 @@ panel_widget_is_cursor(PanelWidget *panel, int overlap)
 	return FALSE;
 }
 
-static void
-emit_background_changed (PanelWidget *panel_widget)
-{
-	g_signal_emit (panel_widget, panel_widget_signals [BACK_CHANGE_SIGNAL], 0);
-}
-
-void
-panel_widget_set_back_pixmap (PanelWidget *panel,
-			      const char  *image_path)
-{
-	g_return_if_fail (PANEL_IS_WIDGET (panel));
-	g_return_if_fail (image_path != NULL);
-
-	if (panel_background_set_image (&panel->background, image_path, FALSE, FALSE, FALSE))
-		emit_background_changed (panel);
-
-	gtk_widget_queue_draw (GTK_WIDGET (panel));
-}
-
-void
-panel_widget_set_back_color (PanelWidget *panel,
-			     PanelColor  *color)
-{
-	g_return_if_fail (PANEL_IS_WIDGET (panel));
-	g_return_if_fail (color != NULL);
-
-	if (panel_background_set_color (&panel->background, color))
-		emit_background_changed (panel);
-
-	gtk_widget_queue_draw (GTK_WIDGET (panel));
-}
-
 static void
 panel_widget_style_set (GtkWidget *widget,
 			GtkStyle  *previous_style)
@@ -1485,7 +1407,7 @@ panel_widget_realize (GtkWidget *widget)
 {
 	PanelWidget *panel = (PanelWidget *) widget;
 
-	g_signal_connect (panel->panel_parent, "configure-event",
+	g_signal_connect (panel->toplevel, "configure-event",
 			  G_CALLBACK (toplevel_configure_event), panel);
 
 	GTK_WIDGET_CLASS (panel_widget_parent_class)->realize (widget);
@@ -1506,7 +1428,7 @@ panel_widget_unrealize (GtkWidget *widget)
 	panel_background_unrealized (&panel->background);
 
 	g_signal_handlers_disconnect_by_func (
-		panel->panel_parent,
+		panel->toplevel,
 		G_CALLBACK (toplevel_configure_event),
 		panel);
 
@@ -1524,9 +1446,6 @@ panel_widget_finalize (GObject *obj)
 
 	panel_background_free (&panel->background);
 
-	g_free (panel->unique_id);
-	panel->unique_id = NULL;
-
 	G_OBJECT_CLASS (panel_widget_parent_class)->finalize (obj);
 }
 
@@ -1545,74 +1464,6 @@ panel_widget_destroy (GtkObject *obj)
 		GTK_OBJECT_CLASS (panel_widget_parent_class)->destroy (obj);
 }
 
-static gchar * 
-generate_unique_id (void)
-{
-	gint32 id;
-	GTimeVal tv;
-	static int incr = -1;
-	gchar *retval = NULL;
-
-	if (incr == -1)
-		incr = (rand () >> 3) & 0xFF;
-
-	id = (rand () >> 3) && 0xFFF;
-	id += (incr << 12);
-
-	g_get_current_time (&tv);
-	id += (tv.tv_usec & 0x7FF) << 20;
-
-	incr ++;
-
-	retval = g_strdup_printf ("%u", id);	
-
-	if (panel_widget_get_by_id (retval) != NULL) {
-		g_free (retval);
-		retval = generate_unique_id ();
-	}
-
-	return retval;
-}
-
-PanelWidget *
-panel_widget_get_by_id (gchar *id)
-{
-	GSList *li;
-
-	g_return_val_if_fail (id != NULL, NULL);
-
-	for (li = panels; li != NULL; li = li->next) {
-		PanelWidget *panel = li->data;
-
-		if (panel->unique_id && ! strcmp (panel->unique_id, id))
-			return panel;
-	}
-
-	return NULL;
-}
-
-void
-panel_widget_set_id (PanelWidget *panel, const char *id)
-{
-	g_return_if_fail (id != NULL);
-
-	if (panel->unique_id)
-		g_free (panel->unique_id);
-
-	panel->unique_id = g_strdup (id);
-
-	return;
-}
-
-void
-panel_widget_set_new_id (PanelWidget *panel) 
-{
-	if (panel->unique_id)
-		g_free (panel->unique_id);
-
-	panel->unique_id = generate_unique_id ();
-}
-
 static void
 panel_widget_instance_init (PanelWidget *panel)
 {
@@ -1622,7 +1473,6 @@ panel_widget_instance_init (PanelWidget *panel)
 		widget,
 		gtk_widget_get_events (widget) | GDK_BUTTON_RELEASE_MASK);
 	
-	panel->unique_id     = NULL;
 	panel->packed        = FALSE;
 	panel->orient        = GTK_ORIENTATION_HORIZONTAL;
 	panel->thick         = PANEL_MINIMUM_WIDTH;
@@ -1630,8 +1480,6 @@ panel_widget_instance_init (PanelWidget *panel)
 	panel->applet_list   = NULL;
 	panel->master_widget = NULL;
 	panel->drop_widget   = widget;
-	panel->inhibit_draw  = FALSE;
-	panel->delete_dialog = NULL;
 	panel->create_launcher_dialog_list = NULL;
 
 	panel_background_init (&panel->background);
@@ -1640,16 +1488,10 @@ panel_widget_instance_init (PanelWidget *panel)
 }
 
 GtkWidget *
-panel_widget_new (const char          *panel_id,
-		  gboolean             packed,
-		  GtkOrientation       orient,
-		  int                  sz,
-		  PanelBackgroundType  back_type,
-		  const char          *back_pixmap,
-		  gboolean             fit_pixmap_bg,
-		  gboolean             stretch_pixmap_bg,
-		  gboolean             rotate_pixmap_bg,
-		  PanelColor          *back_color)
+panel_widget_new (PanelToplevel  *toplevel,
+		  gboolean        packed,
+		  GtkOrientation  orient,
+		  int             sz)
 {
 	PanelWidget *panel;
 
@@ -1658,16 +1500,6 @@ panel_widget_new (const char          *panel_id,
         GTK_WIDGET_UNSET_FLAGS (panel, GTK_NO_WINDOW);
         GTK_WIDGET_SET_FLAGS (panel, GTK_CAN_FOCUS);
 
-	if (!panel_id)
-		panel->unique_id = generate_unique_id ();
-	else
-		panel->unique_id = g_strdup (panel_id);
-
-	if (panel_background_set (
-		&panel->background, back_type, back_color, back_pixmap,
-		fit_pixmap_bg, stretch_pixmap_bg, rotate_pixmap_bg))
-		emit_background_changed (panel);
-
 	panel->orient = orient;
 	panel->sz = sz;
 
@@ -1676,6 +1508,9 @@ panel_widget_new (const char          *panel_id,
 		panel->size = 0;
 	else
 		panel->size = G_MAXINT;
+
+	panel->toplevel    = toplevel;
+	panel->drop_widget = GTK_WIDGET (toplevel);
 	
 	return GTK_WIDGET (panel);
 }
@@ -1709,7 +1544,7 @@ panel_widget_applet_drag_start_no_grab (PanelWidget *panel,
 #endif
 	panel->currently_dragged_applet = ad;
 	if (drag_off == PW_DRAG_OFF_CURSOR)
-		ad->drag_off = panel_widget_get_cursorloc (panel) - ad->pos;
+		ad->drag_off = panel_widget_get_cursorloc (panel) - ad->constrained;
 	else if (drag_off == PW_DRAG_OFF_CENTER)
 		ad->drag_off = ad->cells / 2;
 	else
@@ -1743,8 +1578,9 @@ panel_widget_applet_drag_end_no_grab (PanelWidget *panel)
 
 void
 panel_widget_applet_drag_start (PanelWidget *panel,
-				GtkWidget *applet,
-				int drag_off)
+				GtkWidget   *applet,
+				int          drag_off,
+				guint32      time_)
 {
 	g_return_if_fail (PANEL_IS_WIDGET (panel));
 	g_return_if_fail (GTK_IS_WIDGET (applet));
@@ -1764,7 +1600,7 @@ panel_widget_applet_drag_start (PanelWidget *panel,
 
 		status = gdk_pointer_grab (applet->window, FALSE,
 					   APPLET_EVENT_MASK, NULL,
-					   fleur_cursor, GDK_CURRENT_TIME);
+					   fleur_cursor, time_);
 
 		gdk_cursor_unref (fleur_cursor);
 		gdk_flush ();
@@ -1820,16 +1656,16 @@ walk_up_to (int pos, GList *list)
 
 	ad = list->data;
 
-	if (ad->pos <= pos &&
-	    ad->pos + ad->cells > pos)
+	if (ad->constrained <= pos &&
+	    ad->constrained + ad->cells > pos)
 		return list;
 	while (list->next != NULL &&
-	       ad->pos + ad->cells <= pos) {
+	       ad->constrained + ad->cells <= pos) {
 		list = list->next;
 		ad = list->data;
 	}
 	while (list->prev != NULL &&
-	       ad->pos > pos) {
+	       ad->constrained > pos) {
 		list = list->prev;
 		ad = list->data;
 	}
@@ -1841,8 +1677,8 @@ is_in_applet (int pos, AppletData *ad)
 {
 	g_return_val_if_fail (ad != NULL, NULL);
 
-	if (ad->pos <= pos &&
-	    ad->pos + ad->min_cells > pos)
+	if (ad->constrained <= pos &&
+	    ad->constrained + ad->min_cells > pos)
 		return ad->applet;
 	return NULL;
 }
@@ -1858,7 +1694,7 @@ panel_widget_get_free_spot (PanelWidget *panel, AppletData *ad, int place)
 	g_return_val_if_fail (PANEL_IS_WIDGET (panel), -1);
 	g_return_val_if_fail (ad != NULL, -1);
 
-	if (ad->pos >= panel->size)
+	if (ad->constrained >= panel->size)
 		return -1;
 
 	if (panel->applet_list == NULL) {
@@ -1935,8 +1771,7 @@ panel_widget_nice_move (PanelWidget *panel, AppletData *ad, int pos)
 	g_return_if_fail (PANEL_IS_WIDGET (panel));
 	g_return_if_fail (ad != NULL);
 
-	if (pos < 0 ||
-	    pos == ad->pos)
+	if (pos < 0 || pos == ad->pos)
 		return;
 
 	l = g_list_find (panel->applet_list, ad);
@@ -1944,7 +1779,7 @@ panel_widget_nice_move (PanelWidget *panel, AppletData *ad, int pos)
 		pad1 = l->prev->data;
 	}
 
-	ad->pos = pos;
+	ad->pos = ad->constrained = pos;
 
 	panel->applet_list =
 		panel_g_list_resort_item (panel->applet_list, ad,
@@ -1964,6 +1799,8 @@ panel_widget_nice_move (PanelWidget *panel, AppletData *ad, int pos)
 	if (pad2 && pad2 != pad1 && pad2->expand_major) {
 		panel_widget_queue_applet_for_resize (pad2);
 	}
+
+	emit_applet_moved (panel, ad);
 }
 
 /* schedule to run the below function */
@@ -1988,7 +1825,7 @@ panel_widget_applet_move_to_cursor (PanelWidget *panel)
 
 	ad = panel->currently_dragged_applet;
 
-	pos = ad->pos;
+	pos = ad->constrained;
 
 	applet = ad->applet;
 	g_assert(GTK_IS_WIDGET(applet));
@@ -2017,11 +1854,13 @@ panel_widget_applet_move_to_cursor (PanelWidget *panel)
 
 				/*disable reentrancy into this function*/
 				if (!panel_widget_reparent (panel, new_panel, applet, pos)) {
-					panel_widget_applet_drag_start (panel, applet, ad->drag_off);
+					panel_widget_applet_drag_start (
+						panel, applet, ad->drag_off, GDK_CURRENT_TIME);
 					continue;
 				}
 
-				panel_widget_applet_drag_start (new_panel, applet, ad->drag_off);
+				panel_widget_applet_drag_start (
+					new_panel, applet, ad->drag_off, GDK_CURRENT_TIME);
 				schedule_try_move (new_panel, TRUE);
 
 				return;
@@ -2145,8 +1984,8 @@ panel_widget_applet_event(GtkWidget *widget, GdkEvent *event)
 			if ( ! commie_mode &&
 			    bevent->button == 2) {
 				/* Start drag */
-				panel_widget_applet_drag_start
-					(panel, widget, PW_DRAG_OFF_CURSOR);
+				panel_widget_applet_drag_start (
+					panel, widget, PW_DRAG_OFF_CURSOR, bevent->time);
 				return TRUE;
 			}
 
@@ -2346,9 +2185,7 @@ int
 panel_widget_add (PanelWidget *panel,
 		  GtkWidget   *applet,
 		  int          pos,
-		  gboolean     insert_at_pos,
-		  gboolean     expand_major,
-		  gboolean     expand_minor)
+		  gboolean     insert_at_pos)
 {
 	AppletData *ad = NULL;
 
@@ -2391,11 +2228,12 @@ panel_widget_add (PanelWidget *panel,
 		ad->cells = 1;
 		ad->min_cells = 1;
 		ad->pos = pos;
+		ad->constrained = pos;
 		ad->drag_off = 0;
 		ad->dirty = FALSE;
 		ad->no_die = 0;
-		ad->expand_major = expand_major;
-		ad->expand_minor = expand_minor;
+		ad->expand_major = FALSE;
+		ad->expand_minor = FALSE;
 		ad->size_hints = NULL;
 		g_object_set_data (G_OBJECT (applet),
 				   PANEL_APPLET_DATA, ad);
@@ -2437,7 +2275,6 @@ panel_widget_reparent (PanelWidget *old_panel,
 {
 	AppletData *ad;
 	GtkWidget *focus_widget = NULL;
-	AppletInfo* info;
 
 	g_return_val_if_fail(PANEL_IS_WIDGET(old_panel),-1);
 	g_return_val_if_fail(PANEL_IS_WIDGET(new_panel),-1);
@@ -2450,29 +2287,21 @@ panel_widget_reparent (PanelWidget *old_panel,
 	/* Don't try and reparent to an explicitly hidden panel,
 	 * very confusing for the user ...
 	 */
-	if (BASEP_IS_WIDGET (new_panel->panel_parent) &&
-	    (BASEP_WIDGET (new_panel->panel_parent)->state == BASEP_HIDDEN_LEFT ||
-	     BASEP_WIDGET (new_panel->panel_parent)->state == BASEP_HIDDEN_RIGHT))
+	if (panel_toplevel_get_is_hidden (new_panel->toplevel))
 		return FALSE;
-
-	info = g_object_get_data (G_OBJECT (ad->applet), "applet_info");
-
+	
 	/*we'll resize both panels anyway*/
 	ad->dirty = FALSE;
 	
-	ad->pos = pos;
+	ad->pos = ad->constrained = pos;
 
 	ad->no_die++;
 
 	panel_widget_reset_saved_focus (old_panel);
-	if (GTK_CONTAINER (old_panel)->focus_child == applet) {
-		focus_widget = gtk_window_get_focus (GTK_WINDOW (old_panel->panel_parent));
-	}
+	if (GTK_CONTAINER (old_panel)->focus_child == applet)
+		focus_widget = gtk_window_get_focus (GTK_WINDOW (old_panel->toplevel));
 	gtk_widget_reparent (applet, GTK_WIDGET (new_panel));
 
-	if (info && info->type == APPLET_BONOBO)
-		panel_applet_frame_set_panel (PANEL_APPLET_FRAME (ad->applet), new_panel);
-
 	if (GTK_WIDGET_CAN_FOCUS (new_panel))
 		GTK_WIDGET_UNSET_FLAGS (new_panel, GTK_CAN_FOCUS);
 	if (focus_widget) {
@@ -2484,58 +2313,51 @@ panel_widget_reparent (PanelWidget *old_panel,
 				       GTK_DIR_TAB_FORWARD,
 				       &return_val);
 	}
- 	gtk_window_present (GTK_WINDOW (new_panel->panel_parent));
+ 	gtk_window_present (GTK_WINDOW (new_panel->toplevel));
 
 	gdk_flush();
 
 	ad->no_die--;
 
+	emit_applet_moved (new_panel, ad);
+
 	return TRUE;
 }
 
 void
-panel_widget_change_params(PanelWidget         *panel,
-			   GtkOrientation       orient,
-			   int                  sz,
-			   PanelBackgroundType  back_type,
-			   const char          *pixmap,
-			   gboolean             fit_pixmap_bg,
-			   gboolean             stretch_pixmap_bg,
-			   gboolean             rotate_pixmap_bg,
-			   PanelColor          *back_color)
+panel_widget_set_packed (PanelWidget *panel_widget,
+			 gboolean     packed)
 {
-	GtkOrientation oldorient;
-	int            oldsz;
-	gboolean       change_back = FALSE;
+	panel_widget->packed = packed;
 
-	g_return_if_fail (PANEL_IS_WIDGET (panel));
-	g_return_if_fail (GTK_WIDGET_REALIZED (panel));
+	gtk_widget_queue_resize (GTK_WIDGET (panel_widget));
+}
 
-	oldorient = panel->orient;
-	panel->orient = orient;
+void
+panel_widget_set_orientation (PanelWidget    *panel_widget,
+			      GtkOrientation  orientation)
+{
+	panel_widget->orient = orientation;
 
-	oldsz = panel->sz;
-	panel->sz = sz;
+	gtk_widget_queue_resize (GTK_WIDGET (panel_widget));
+}
+
+void
+panel_widget_set_size (PanelWidget *panel_widget,
+		       int          size)
+{
+	g_return_if_fail (PANEL_IS_WIDGET (panel_widget));
+
+	if (size == panel_widget->sz)
+		return;
+
+	panel_widget->sz = size;
 	
-	queue_resize_on_all_applets (panel);
+	queue_resize_on_all_applets (panel_widget);
 
-	if (oldorient != panel->orient)
-	   	g_signal_emit (
-			panel, panel_widget_signals [ORIENT_CHANGE_SIGNAL], 0);
+	g_signal_emit (panel_widget, panel_widget_signals [SIZE_CHANGE_SIGNAL], 0);
 
-	if (oldsz != panel->sz)
-	   	g_signal_emit (
-			panel, panel_widget_signals [SIZE_CHANGE_SIGNAL], 0);
-
-	change_back = panel_background_set (
-			&panel->background, back_type, back_color, pixmap,
-			fit_pixmap_bg, stretch_pixmap_bg, rotate_pixmap_bg);
-	if (change_back)
-		emit_background_changed (panel);
-
-	/* inhibit draws until we resize */
-	panel->inhibit_draw = TRUE;
-	gtk_widget_queue_resize (GTK_WIDGET (panel));
+	gtk_widget_queue_resize (GTK_WIDGET (panel_widget));
 }
 
 static void 
@@ -2616,7 +2438,7 @@ panel_widget_free_move_applet (PanelWidget      *panel,
 	}
 
 	pos = panel_widget_get_free_spot (panel, ad, 
-                                          increment + ad->pos + ad->drag_off);
+                                          increment + ad->constrained + ad->drag_off);
 	panel_widget_nice_move (panel, ad, pos);
 }
 
@@ -2683,53 +2505,55 @@ panel_widget_real_focus (GtkWidget        *widget,
 }
 
 void 
-panel_widget_focus (PanelWidget *panel)
+panel_widget_focus (PanelWidget *panel_widget)
 {
+	if (panel_toplevel_get_is_attached (panel_widget->toplevel))
+		return;
+
 	/*
          * Set the focus back on the panel; we unset the focus child so that
 	 * the next time focus is inside the panel we do not remember the
 	 * previously focused child. We also need to set GTK_CAN_FOCUS flag
 	 * on the panel as it is unset when this function is called.
 	 */
-	if (!DRAWER_IS_WIDGET (panel->panel_parent)) {
-		gtk_container_set_focus_child (GTK_CONTAINER (panel), NULL);
-		GTK_WIDGET_SET_FLAGS (panel, GTK_CAN_FOCUS);
-		gtk_widget_grab_focus (GTK_WIDGET (panel));
-	}
+	gtk_container_set_focus_child (GTK_CONTAINER (panel_widget), NULL);
+	GTK_WIDGET_SET_FLAGS (panel_widget, GTK_CAN_FOCUS);
+	gtk_widget_grab_focus (GTK_WIDGET (panel_widget));
 }
 
 
-PanelOrient
-panel_widget_get_applet_orient (PanelWidget *panel)
+PanelOrientation
+panel_widget_get_applet_orientation (PanelWidget *panel)
 {
-	g_return_val_if_fail (PANEL_IS_WIDGET (panel), PANEL_ORIENT_UP);
-	g_return_val_if_fail (panel->panel_parent != NULL, PANEL_ORIENT_UP);
+	g_return_val_if_fail (PANEL_IS_WIDGET (panel), PANEL_ORIENTATION_TOP);
+	g_return_val_if_fail (PANEL_IS_TOPLEVEL (panel->toplevel), PANEL_ORIENTATION_TOP);
 
-	if (!BASEP_IS_WIDGET (panel->panel_parent))
-		return PANEL_ORIENT_DOWN;
-
-	return basep_widget_get_applet_orient (BASEP_WIDGET (panel->panel_parent));
+	return panel_toplevel_get_orientation (panel->toplevel);
 }
 
 void
-panel_widget_remove_drawers (PanelWidget *panel_widget)
+panel_widget_set_applet_expandable (PanelWidget *panel,
+				    GtkWidget   *applet,
+				    gboolean     major,
+				    gboolean     minor)
 {
-	GList *l;
+	AppletData *ad;
 
-	for (l = panel_widget->applet_list; l ; l = l->next) {
+	ad = g_object_get_data (G_OBJECT (applet), PANEL_APPLET_DATA);
+	if (!ad)
+		return;
 
-		AppletData* applet_data = l->data;
-		AppletInfo* info = g_object_get_data (
-						G_OBJECT (applet_data->applet), "applet_info");
+	if (ad->expand_major == major && ad->expand_minor == minor)
+		return;
 
-		if (info && info->type == APPLET_DRAWER) {
-			Drawer *drawer = (Drawer *) info->data;
-			BasePWidget* drawer_widget = (BASEP_WIDGET (drawer->drawer));
-
-			panel_widget_remove_drawers (PANEL_WIDGET(drawer_widget->panel));
-
-			panel_remove_from_gconf(PANEL_WIDGET(drawer_widget->panel));
-		}
-	}
+	ad->expand_major = major;
+	ad->expand_minor = minor;
 
+	gtk_widget_queue_resize (GTK_WIDGET (panel));
+}
+
+gboolean
+panel_applet_is_in_drag ()
+{
+	return panel_applet_in_drag;
 }
