@@ -32,7 +32,6 @@
 #include "drawer.h"
 #include "button-widget.h"
 #include "distribution.h"
-#include "gnome-run.h"
 #include "launcher.h"
 #include "menu-fentry.h"
 #include "menu-util.h"
@@ -191,9 +190,6 @@ panel_applet_added(GtkWidget *widget, GtkWidget *applet, gpointer data)
 	toplevel = PANEL_WIDGET (widget)->toplevel;
 	info = g_object_get_data (G_OBJECT (applet), "applet_info");
 
-	panel_toplevel_unhide (toplevel);
-	panel_toplevel_queue_auto_hide (toplevel);
-
 	orientation_change(info,PANEL_WIDGET(widget));
 	size_change(info,PANEL_WIDGET(widget));
 #ifdef FIXME_FOR_NEW_CONFIG
@@ -402,7 +398,8 @@ panel_key_press_event (GtkWidget   *widget,
 	 * bindings do not work. We get around this by
 	 * activating the key bindings here.
 	 */ 
-	if (GTK_IS_SOCKET (GTK_WINDOW (widget)->focus_widget))
+	if (GTK_IS_SOCKET (GTK_WINDOW (widget)->focus_widget) &&
+	    event->keyval == GDK_F10 && event->state == GDK_CONTROL_MASK)
 		return gtk_bindings_activate (GTK_OBJECT (widget),
 					      event->keyval,
 					      event->state);
@@ -618,8 +615,10 @@ drop_urilist (PanelWidget *panel, int pos, char *urilist)
 }
 
 static void
-drop_internal_icon (PanelWidget *panel, int pos, const char *icon_name,
-		    int action)
+drop_internal_icon (PanelWidget *panel,
+		    int          pos,
+		    const char  *icon_name,
+		    int          action)
 {
 	Launcher *old_launcher = NULL;
 
@@ -629,7 +628,7 @@ drop_internal_icon (PanelWidget *panel, int pos, const char *icon_name,
 	if (action == GDK_ACTION_MOVE)
 		old_launcher = find_launcher (icon_name);
 	
-	panel_launcher_create (panel->toplevel, pos, icon_name);
+	panel_launcher_create_copy (panel->toplevel, pos, icon_name);
 
 	if (old_launcher && old_launcher->button) {
 		if (old_launcher->prop_dialog) {
@@ -1119,8 +1118,6 @@ panel_delete_without_query (PanelToplevel *toplevel)
 
 	panel_widget = panel_toplevel_get_panel_widget (toplevel);
 
-	/* FIXME_FOR_NEW_TOPLEVEL: make sure we are deleting all applets on the panel */
-
 	if (panel_toplevel_get_is_attached (toplevel) &&
 	    panel_widget->master_widget) {
 		AppletInfo *info;
